from factorialhr.endpoints import _base


class Membership(_base.Endpoint):
    endpoint = '/2025-01-01/resources/teams/memberships'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-teams-memberships."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-teams-memberships."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, membership_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-teams-memberships-id."""
        return await self.api.delete(f'{self.endpoint}/{membership_id}', **kwargs)

    async def update(self, *, membership_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-teams-memberships-id."""
        return await self.api.put(f'{self.endpoint}/{membership_id}', **kwargs)

    async def single(self, *, membership_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-teams-memberships-id."""
        return await self.api.get(f'{self.endpoint}/{membership_id}', **kwargs)


class Team(_base.Endpoint):
    endpoint = '/2025-01-01/resources/teams/teams'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-teams-teams."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-teams-teams."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, team_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-teams-teams-id."""
        return await self.api.delete(f'{self.endpoint}/{team_id}', **kwargs)

    async def update(self, *, team_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-teams-teams-id."""
        return await self.api.put(f'{self.endpoint}/{team_id}', **kwargs)

    async def single(self, *, team_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-teams-teams-id."""
        return await self.api.get(f'{self.endpoint}/{team_id}', **kwargs)



__version__ = '3.0.1a2'

from factorialhr.auth import AccessTokenAuth, ApiKeyAuth, RefreshTokenAuth
from factorialhr.client import ApiClient

__all__ = ['AccessTokenAuth', 'ApiClient', 'ApiKeyAuth', 'RefreshTokenAuth']



from factorialhr.endpoints import _base


class BreakConfiguration(_base.Endpoint):
    endpoint = '/2025-01-01/resources/attendance/break_configurations'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-break-configurations."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-break-configurations."""
        return await self.api.post(self.endpoint, **kwargs)

    async def update(self, *, break_configuration_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-attendance-break-configurations-id."""
        return await self.api.put(f'{self.endpoint}/{break_configuration_id}', **kwargs)

    async def single(self, *, break_configuration_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-break-configurations-id."""
        return await self.api.get(f'{self.endpoint}/{break_configuration_id}', **kwargs)


class EditTimesheetRequest(_base.Endpoint):
    endpoint = '/2025-01-01/resources/attendance/edit_timesheet_requests'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-edit-timesheet-requests."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-edit-timesheet-requests."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, edit_time_sheet_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-attendance-edit-timesheet-requests-id."""
        return await self.api.delete(f'{self.endpoint}/{edit_time_sheet_request_id}', **kwargs)

    async def update(self, *, edit_time_sheet_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-attendance-edit-timesheet-requests-id."""
        return await self.api.put(f'{self.endpoint}/{edit_time_sheet_request_id}', **kwargs)

    async def single(self, *, edit_time_sheet_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-edit-timesheet-requests-id."""
        return await self.api.get(f'{self.endpoint}/{edit_time_sheet_request_id}', **kwargs)


class EstimatedTimes(_base.Endpoint):
    endpoint = '/2025-01-01/resources/attendance/estimated_times'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-estimated-times."""
        return await self.api.get(self.endpoint, **kwargs)


class OpenShifts(_base.Endpoint):
    endpoint = '/2025-01-01/resources/attendance/open_shifts'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-open-shifts."""
        return await self.api.get(self.endpoint, **kwargs)


class OverTimeRequest(_base.Endpoint):
    endpoint = '/2025-01-01/resources/attendance/overtime_requests'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-overtime-requests."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-overtime-requests."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, over_time_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-attendance-overtime-requests-id."""
        return await self.api.delete(f'{self.endpoint}/{over_time_request_id}', **kwargs)

    async def update(self, *, over_time_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-attendance-overtime-requests-id."""
        return await self.api.put(f'{self.endpoint}/{over_time_request_id}', **kwargs)

    async def single(self, *, over_time_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-overtime-requests-id."""
        return await self.api.get(f'{self.endpoint}/{over_time_request_id}', **kwargs)

    async def approve(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-overtime-requests-approve."""
        return await self.api.post(self.endpoint, **kwargs)

    async def reject(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-overtime-requests-reject."""
        return await self.api.post(self.endpoint, **kwargs)


class Shift(_base.Endpoint):
    endpoint = '/2025-01-01/resources/attendance/shifts'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-shifts."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-shifts."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, over_time_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-attendance-shifts-id."""
        return await self.api.delete(f'{self.endpoint}/{over_time_request_id}', **kwargs)

    async def update(self, *, over_time_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-attendance-shifts-id."""
        return await self.api.put(f'{self.endpoint}/{over_time_request_id}', **kwargs)

    async def single(self, *, over_time_request_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-shifts-id."""
        return await self.api.get(f'{self.endpoint}/{over_time_request_id}', **kwargs)

    async def autofill(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-shifts-autofill."""
        return await self.api.post(self.endpoint, **kwargs)

    async def break_end(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-shifts-break-end."""
        return await self.api.post(self.endpoint, **kwargs)

    async def break_start(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-shifts-break-start."""
        return await self.api.post(self.endpoint, **kwargs)

    async def clock_in(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-shifts-clock-in."""
        return await self.api.post(self.endpoint, **kwargs)

    async def clock_out(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-shifts-clock-out."""
        return await self.api.post(self.endpoint, **kwargs)

    async def clock_in_out(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-attendance-shifts-toggle-clock."""
        return await self.api.post(self.endpoint, **kwargs)


class WorkedTime(_base.Endpoint):
    endpoint = '/2025-01-01/resources/attendance/worked_times'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-attendance-worked-times."""
        return await self.api.get(self.endpoint, **kwargs)



from factorialhr.endpoints import _base


class PlanningVersion(_base.Endpoint):
    endpoint = '/2025-01-01/resources/time_planning/planning_versions'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-time-planning-planning-versions."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-time-planning-planning-versions."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, planning_version_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-time-planning-planning-versions-id."""
        return await self.api.delete(f'{self.endpoint}/{planning_version_id}', **kwargs)

    async def update(self, *, planning_version_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-time-planning-planning-versions-id."""
        return await self.api.put(f'{self.endpoint}/{planning_version_id}', **kwargs)

    async def bulk_create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-time-planning-planning-versions-bulk-create."""
        return await self.api.post(f'{self.endpoint}/bulk_create', **kwargs)



from factorialhr.endpoints import _base


class ExpenseRecord(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/expense_records'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-expense-records."""
        return await self.api.get(self.endpoint, **kwargs)

    async def single(self, *, expense_record_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-expense-records-id."""
        return await self.api.get(f'{self.endpoint}/{expense_record_id}', **kwargs)


class ExportableExpense(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/exportable_expenses'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-exportable-expenses."""
        return await self.api.get(self.endpoint, **kwargs)


class ExportableProject(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/exportable_projects'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-exportable-projects."""
        return await self.api.get(self.endpoint, **kwargs)


class FlexibleTimeRecord(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/flexible_time_records'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-flexible-time-records."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-flexible-time-records."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, flexible_time_record_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-project-management-flexible-time-records-id."""
        return await self.api.delete(f'{self.endpoint}/{flexible_time_record_id}', **kwargs)

    async def update(self, *, flexible_time_record_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-project-management-flexible-time-records-id."""
        return await self.api.put(f'{self.endpoint}/{flexible_time_record_id}', **kwargs)

    async def single(self, *, flexible_time_record_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-flexible-time-records-id."""
        return await self.api.get(f'{self.endpoint}/{flexible_time_record_id}', **kwargs)


class FlexibleTimeRecordComment(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/flexible_time_record_comments'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-flexible-time-record-comments."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-flexible-time-record-comments."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete_by_flexible_time_record(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-flexible-time-record-comments-delete-by-flexible-time-record."""
        return await self.api.post(f'{self.endpoint}/delete_by_flexible_time_record', **kwargs)

    async def update_by_flexible_time_record(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-flexible-time-record-comments-update-by-flexible-time-record."""
        return await self.api.post(f'{self.endpoint}/update_by_flexible_time_record', **kwargs)

    async def single(self, *, flexible_time_record_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-flexible-time-record-comments-id."""
        return await self.api.get(f'{self.endpoint}/{flexible_time_record_id}', **kwargs)


class Project(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/projects'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-projects."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-projects."""
        return await self.api.post(self.endpoint, **kwargs)

    async def update(self, *, project_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-project-management-projects-id."""
        return await self.api.post(f'{self.endpoint}/{project_id}', **kwargs)

    async def single(self, *, project_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-projects-id."""
        return await self.api.get(f'{self.endpoint}/{project_id}', **kwargs)

    async def activate(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-projects-activate."""
        return await self.api.post(f'{self.endpoint}/activate', **kwargs)

    async def change_assignment(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-projects-change-assignment."""
        return await self.api.post(f'{self.endpoint}/change_assignment', **kwargs)

    async def close(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-projects-close."""
        return await self.api.post(f'{self.endpoint}/close', **kwargs)

    async def soft_delete(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-projects-soft-delete."""
        return await self.api.post(f'{self.endpoint}/soft_delete', **kwargs)


class ProjectTask(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/project_tasks'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-project-tasks."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-project-tasks."""
        return await self.api.post(self.endpoint, **kwargs)

    async def update(self, *, project_task_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-project-management-project-tasks-id."""
        return await self.api.post(f'{self.endpoint}/{project_task_id}', **kwargs)

    async def single(self, *, project_task_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-project-tasks-id."""
        return await self.api.get(f'{self.endpoint}/{project_task_id}', **kwargs)

    async def bulk_destroy(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-project-tasks-bulk-destroy."""
        return await self.api.post(f'{self.endpoint}/bulk_destroy', **kwargs)

    async def bulk_duplicate(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-project-tasks-bulk-duplicate."""
        return await self.api.post(f'{self.endpoint}/bulk_duplicate', **kwargs)


class ProjectWorker(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/project_workers'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-project-workers."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-project-workers."""
        return await self.api.post(self.endpoint, **kwargs)

    async def single(self, *, project_worker_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-project-workers-id."""
        return await self.api.get(f'{self.endpoint}/{project_worker_id}', **kwargs)

    async def bulk_assign(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-project-workers-bulk-assign."""
        return await self.api.post(f'{self.endpoint}/bulk_assign', **kwargs)

    async def bulk_create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-project-workers-bulk-create."""
        return await self.api.post(f'{self.endpoint}/bulk_create', **kwargs)

    async def unassign(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-project-workers-unassign."""
        return await self.api.post(f'{self.endpoint}/unassign', **kwargs)


class Subproject(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/subprojects'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-subprojects."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-subprojects."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, subproject_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-project-management-subprojects-id."""
        return await self.api.delete(f'{self.endpoint}/{subproject_id}', **kwargs)

    async def rename(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-subprojects-rename."""
        return await self.api.post(f'{self.endpoint}/rename', **kwargs)

    async def single(self, *, subproject_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-subprojects-id."""
        return await self.api.get(f'{self.endpoint}/{subproject_id}', **kwargs)


class TimeRecord(_base.Endpoint):
    endpoint = '/2025-01-01/resources/project_management/time_records'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-time-records."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-time-records."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, flexible_time_record_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-project-management-time-records-id."""
        return await self.api.delete(f'{self.endpoint}/{flexible_time_record_id}', **kwargs)

    async def single(self, *, flexible_time_record_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-project-management-time-records-id."""
        return await self.api.get(f'{self.endpoint}/{flexible_time_record_id}', **kwargs)

    async def bulk_delete(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-time-records-bulk-delete."""
        return await self.api.post(f'{self.endpoint}/bulk_delete', **kwargs)

    async def bulk_process(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-time-records-bulk-process."""
        return await self.api.post(f'{self.endpoint}/bulk_process', **kwargs)

    async def update_project_worker(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-project-management-time-records-update-project-worker."""
        return await self.api.post(f'{self.endpoint}/update_project_worker', **kwargs)



import time
import typing

import httpx


class ApiKeyAuth(httpx.Auth):
    """Authorization using an api key."""

    def __init__(self, api_key: str):
        self.api_key = api_key

    def auth_flow(self, request: httpx.Request):
        """Implement the authentication flow."""
        request.headers['x-api-key'] = self.api_key
        yield request


class AccessTokenAuth(httpx.Auth):
    """Authorization using an access token. Not refreshing the session."""

    def __init__(self, access_token: str, token_type: str = 'Bearer'):  # noqa: S107
        self.access_token = access_token
        self.token_type = token_type

    def auth_flow(self, request: httpx.Request):
        """Implement the authentication flow."""
        request.headers['Authorization'] = f'{self.token_type} {self.access_token}'
        yield request


class RefreshTokenAuth(httpx.Auth):
    """Authorization using a refresh token which refreshes the access key automatically."""

    def __init__(  # noqa: PLR0913
        self,
        refresh_token: str,
        client_id: str,
        client_secret: str,
        access_token: str,
        token_type: str,
        expires_at: int,
    ):
        self.refresh_token = refresh_token
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token = access_token
        self.expires_at = expires_at
        self.token_type = token_type
        self._client = httpx.AsyncClient()

    async def async_auth_flow(self, request: httpx.Request) -> typing.AsyncGenerator[httpx.Request, httpx.Response]:
        """Implement the authentication flow."""
        if self.expires_at <= time.time():  # token expired
            await self.refresh_access_token(f'{request.url.scheme}://{request.url.host}')

        request.headers['Authorization'] = f'{self.token_type} {self.access_token}'
        yield request

    async def refresh_access_token(self, target_url: str):
        """Refresh the access token."""
        response = await self._client.post(
            f'{target_url}/oauth/token',
            data={
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'grant_type': 'refresh_token',
                'refresh_token': self.refresh_token,
            },
        )
        response.raise_for_status()
        data = response.json()
        self.access_token = data.get('access_token')
        self.token_type = data.get('token_type')
        self.refresh_token = data.get('refresh_token')
        self.expires_at = int(data.get('expires_in')) + int(data.get('created_at'))



import asyncio
import math
import typing

import httpx


class ApiClient:
    """Factorial api class."""

    def __init__(self, base_url: str = 'https://api.factorialhr.com', *, auth: httpx.Auth, **kwargs):
        headers = {'accept': 'application/json'}
        self._client = httpx.AsyncClient(base_url=f'{base_url}/api', headers=headers, auth=auth, **kwargs)

    async def close(self):
        """Close the client session."""
        await self._client.aclose()

    async def __aexit__(self, *_, **__):
        await self.close()

    async def __aenter__(self) -> 'ApiClient':
        await self._client.__aenter__()
        return self

    @staticmethod
    def _eval_http_method(response: httpx.Response, **kwargs) -> typing.Any:
        response.raise_for_status()
        return response.json(**kwargs)

    async def get(self, endpoint: str, **kwargs) -> typing.Any:
        """Perform a get request."""
        resp = await self._client.get(endpoint, **kwargs)
        return self._eval_http_method(resp)

    async def get_all(self, endpoint: str, **kwargs) -> list:
        """Get all data from an endpoint via offset pagination.

        Depending on the amount of objects to query, you might want to increase the timeout by using
        `timeout=httpx.Timeout(...)`.
        More information at https://apidoc.factorialhr.com/docs/how-does-it-work#offset-pagination.
        """
        data, meta = (await self.get(endpoint, **kwargs)).values()
        if not meta['has_next_page']:
            return data
        page_count = math.ceil(meta['total'] / meta['limit'])
        requests = []
        data = []
        for i in range(page_count):
            query_params = kwargs.get('params', {}).copy()
            query_params['page'] = i + 1
            kwargs['params'] = query_params
            requests.append(self.get(endpoint, **kwargs))
        for response in await asyncio.gather(*requests):
            data.extend(response['data'])
        assert meta['total'] == len(data), f'Got only {len(data)} instead of {meta["total"]} items'
        return data

    async def post(self, endpoint: str, **kwargs) -> typing.Any:
        """Perform a post request."""
        resp = await self._client.post(endpoint, **kwargs)
        return self._eval_http_method(resp)

    async def put(self, endpoint: str, **kwargs) -> typing.Any:
        """Perform a put request."""
        resp = await self._client.put(endpoint, **kwargs)
        return self._eval_http_method(resp)

    async def delete(self, endpoint: str, **kwargs) -> typing.Any:
        """Perform a delete request."""
        resp = await self._client.delete(endpoint, **kwargs)
        return self._eval_http_method(resp)



from factorialhr.endpoints import _base


class Employee(_base.Endpoint):
    endpoint = '/2025-01-01/resources/employees/employees'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-employees-employees."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create_with_contracts(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-employees-employees-create-with-contract."""
        return await self.api.post(self.endpoint, **kwargs)

    async def update(self, *, employee_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-employees-employees-id."""
        return await self.api.put(f'{self.endpoint}/{employee_id}', **kwargs)

    async def single(self, *, employee_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-employees-employees-id."""
        return await self.api.get(f'{self.endpoint}/{employee_id}', **kwargs)

    async def invite(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-employees-employees-invite."""
        return await self.api.post(self.endpoint, **kwargs)

    async def terminate(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-employees-employees-terminate."""
        return await self.api.post(self.endpoint, **kwargs)

    async def unterminate(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-employees-employees-unterminate."""
        return await self.api.post(self.endpoint, **kwargs)



from factorialhr.endpoints import _base


class Answer(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/answers'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-answers."""
        return await self.api.get(self.endpoint, **kwargs)

    async def single(self, *, answer_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-answers-id."""
        return await self.api.get(f'{self.endpoint}/{answer_id}', **kwargs)


class Application(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/applications'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-applications."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-applications."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, application_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-ats-applications-id."""
        return await self.api.delete(f'{self.endpoint}/{application_id}', **kwargs)

    async def update(self, *, application_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-ats-applications-id."""
        return await self.api.put(f'{self.endpoint}/{application_id}', **kwargs)

    async def single(self, *, application_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-applications-id."""
        return await self.api.get(f'{self.endpoint}/{application_id}', **kwargs)

    async def apply(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-applications-apply."""
        return await self.api.post(self.endpoint, **kwargs)


class ApplicationPhase(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/application_phases'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-application-phases."""
        return await self.api.get(self.endpoint, **kwargs)

    async def single(self, *, application_phase: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-application-phases-id."""
        return await self.api.get(f'{self.endpoint}/{application_phase}', **kwargs)


class Candidate(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/candidates'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-candidates."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-candidates."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, candidate_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-ats-candidates-id."""
        return await self.api.delete(f'{self.endpoint}/{candidate_id}', **kwargs)

    async def update(self, *, candidate_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-ats-candidates-id."""
        return await self.api.put(f'{self.endpoint}/{candidate_id}', **kwargs)

    async def single(self, *, candidate_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-candidates-id."""
        return await self.api.get(f'{self.endpoint}/{candidate_id}', **kwargs)


class CandidateSource(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/candidate_sources'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-candidate-sources."""
        return await self.api.get(self.endpoint, **kwargs)

    async def single(self, *, candidate_source_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-candidate-sources-id."""
        return await self.api.get(f'{self.endpoint}/{candidate_source_id}', **kwargs)


class EvaluationForm(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/evaluation_forms'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-evaluation-forms."""
        return await self.api.get(self.endpoint, **kwargs)

    async def single(self, *, evaluation_form_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-evaluation-forms-id."""
        return await self.api.get(f'{self.endpoint}/{evaluation_form_id}', **kwargs)

    async def save_as_template(self, *, evaluation_form_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-evaluation-forms-save-as-template."""
        return await self.api.put(f'{self.endpoint}/{evaluation_form_id}', **kwargs)


class Feedback(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/feedbacks'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-feedbacks."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-feedbacks."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, feedback_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-ats-feedbacks-id."""
        return await self.api.delete(f'{self.endpoint}/{feedback_id}', **kwargs)

    async def update(self, *, feedback_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-ats-feedbacks-id."""
        return await self.api.put(f'{self.endpoint}/{feedback_id}', **kwargs)

    async def single(self, *, feedback_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-feedbacks-id."""
        return await self.api.get(f'{self.endpoint}/{feedback_id}', **kwargs)


class HiringStages(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/hiring_stages'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-hiring-stages."""
        return await self.api.get(self.endpoint, **kwargs)

    async def single(self, *, hiring_stage_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-hiring-stages-id."""
        return await self.api.get(f'{self.endpoint}/{hiring_stage_id}', **kwargs)


class JobPosting(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/job_postings'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-job-postings."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-job-postings."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, job_posting_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-ats-job-postings-id."""
        return await self.api.delete(f'{self.endpoint}/{job_posting_id}', **kwargs)

    async def update(self, *, job_posting_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-ats-job-postings-id."""
        return await self.api.put(f'{self.endpoint}/{job_posting_id}', **kwargs)

    async def single(self, *, job_posting_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-job-postings-id."""
        return await self.api.get(f'{self.endpoint}/{job_posting_id}', **kwargs)

    async def duplicate(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-job-postings-duplicate."""
        return await self.api.post(f'{self.endpoint}/duplicate', **kwargs)


class Message(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/messages'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-messages."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-ats-messages."""
        return await self.api.post(self.endpoint, **kwargs)

    async def single(self, *, evaluation_form_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-messages-id."""
        return await self.api.get(f'{self.endpoint}/{evaluation_form_id}', **kwargs)


class RejectReason(_base.Endpoint):
    endpoint = '/2025-01-01/resources/ats/rejection_reasons'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-rejection-reasons."""
        return await self.api.get(self.endpoint, **kwargs)

    async def single(self, *, reject_reason_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-ats-rejection-reasons-id."""
        return await self.api.get(f'{self.endpoint}/{reject_reason_id}', **kwargs)



from factorialhr.endpoints import _base


class Credentials(_base.Endpoint):
    endpoint = '/2025-01-01/resources/api_public/credentials'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-api-public-credentials."""
        return await self.api.get(self.endpoint, **kwargs)


class WebhookSubscription(_base.Endpoint):
    endpoint = '/2025-01-01/resources/api_public/webhook_subscriptions'

    async def all(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-api-public-webhook-subscriptions."""
        return await self.api.get(self.endpoint, **kwargs)

    async def create(self, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/post_api-2025-01-01-resources-api-public-webhook-subscriptions."""
        return await self.api.post(self.endpoint, **kwargs)

    async def delete(self, *, webhook_subscription_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/delete_api-2025-01-01-resources-api-public-webhook-subscriptions-id."""
        return await self.api.delete(f'{self.endpoint}/{webhook_subscription_id}', **kwargs)

    async def update(self, *, webhook_subscription_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/put_api-2025-01-01-resources-api-public-webhook-subscriptions-id."""
        return await self.api.put(f'{self.endpoint}/{webhook_subscription_id}', **kwargs)

    async def single(self, *, webhook_subscription_id: int, **kwargs) -> dict:
        """Implement https://apidoc.factorialhr.com/reference/get_api-2025-01-01-resources-api-public-webhook-subscriptions-id."""
        return await self.api.get(f'{self.endpoint}/{webhook_subscription_id}', **kwargs)
